/*
 * lab1.c
 */

#include "lcd.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>

char colors[] = "ROVAIT";
int i;
uint16_t temp;

char const* get_color()
{
	if (colors[i] == 'R')
		return "Rosu";
	else if (colors[i] == 'O')
		return "Portocaliu";
	else if (colors[i] == 'V')
		return"Verde";
	else if (colors[i] == 'A')
		return "Albastru";
	else if (colors[i] == 'I')
		return "Violet";
	else 
		return "Temperatura";
}

void print_color()
{
	LCD_writeInstr (LCD_INSTR_clearDisplay);
	
	// linia 1
	char line1[15];
	sprintf(line1, "Temp: %d C ", temp);
	LCD_printAt(0x00, line1);
	
	// linia 2
	char line2[15];
	sprintf(line2, "Mod: %s", get_color());
	LCD_printAt(0x40, line2);

	_delay_ms(500);
}

void port_init()
{
	// configurare porturi folosite
	// led  PB5 → B, PB6 → G, PB7 → R;
	DDRA |= ((1 << PB5) | (1 << PB6) | (1 << PB7));
	
	// temp in PD0 → OUT
	DDRD &= ~(1 << PD0);
	PORTD |= (1 << PD0);
}

void timer_init()
{
	OCR1A = 7812;
	TCCR1B = (1 << WGM12) | (5 << CS10 );
	TIMSK1 |= (1 << OCIE1A);
}

void adc_init()
{
    ADMUX = (1<<REFS0);

    // ADC Enable, prescaler 128
    ADCSRA = (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
}

uint16_t adc_read(uint8_t ch)
{
    // selectare canal conversie
    ch &= 0b00000111;
    ADMUX = (ADMUX & 0xF8)|ch;     

    // porneste conversia
    ADCSRA |= (1<<ADSC);

    // asteapta terminarea conversiei
    while(ADCSRA & (1<<ADSC));

    return (ADC);
}

void set_color_temp()
{
	if (temp <= 15)
		PORTB |= (1 << PB5);// Albastru
	else
		if (temp > 15 && temp <= 25)
		{// Portocaliu
			PORTB |= (1 << PB7);
			PORTB |= (1 << PB6);
		}
		else
			PORTB |= (1 << PB7);// Rosu
}			

ISR(TIMER1_COMPA_vect)
{
	temp = adc_read(3);
	temp = -((temp * 5 / 1024) - 0.5) * 100 / 2;
}

void complex_lcd()
{
	char line[15];
	sprintf(line, "Astept un semn !");
	LCD_printAt(0x00, line);
	LCD_printAt(0x47, ":)");
	
	for(;;)
    {
		// verifica sunet
		if ((PIND & (1 << PD0)) == 1) {
			// schimba modul
			i++;
			if (i == 6)
				i = 0;
		
			// culoare led
			switch(i){
				case 0:// Rosu
					PORTB &= ~(1 << PB5);
					PORTB &= ~(1 << PB6);
					PORTB |= (1 << PB7);
					break;
					
				case 1:// O Portocaliu
					PORTB |= (1 << PB6);
					break;
					
				case 2:// G Verde
					PORTB &= ~(1 << PB7);
					break;
					
				case 3:// Albastru
					PORTB &= ~(1 << PB6);
					PORTB |= (1 << PB5);
					break;
					
				case 4: // Violet
					PORTB |= (1 << PB7);
					break;
					
				case 5: // Temperatura
					PORTB &= ~(1 << PB5);
					PORTB &= ~(1 << PB7);
					
					set_color_temp();
					
					break;		
			}
			// lcd text
			print_color();
		}
	}
}

int main()
{
    LCD_init();
	adc_init();
	port_init();
	timer_init();
	i = -1;
	
	sei();
	
	complex_lcd();
    
    return 0;
}
